<?php

namespace Drp\NativeEventApp\Test\Api;

use Drp\NativeEventApp\Test\Support\InteractsWithFileUploads;
use Drp\NativeEventApp\Test\TestCase;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class HandleFileUploadTest extends TestCase
{
    use InteractsWithFileUploads;

    /** @test */
    public function it_can_store_an_uploaded_file(): void
    {
        Storage::fake(config('native-event-app.files.local.disk'));
        $file = UploadedFile::fake()->image('test');

        $uuid = (string) Str::uuid();
        $url = $this->createSignedUrl($uuid);

        $response = $this->call('PUT', $url, [], [], [], [], $file->get());

        $response->assertStatus(200);
        $response->assertJson([
            'id' => $uuid,
        ]);

        Storage::disk(config('native-event-app.files.local.disk'))
            ->assertExists(config('native-event-app.files.local.path') . '/' .$uuid);
    }

    /** @test */
    public function it_expects_a_valid_signature(): void
    {
        $uuid = (string) Str::uuid();
        $url = route('eventpal.sdk.file-upload', [
            'uuid' => $uuid,
            'expires' => 12345,
            'signature' => 'invalid_signature',
        ]);

        Storage::fake(config('native-event-app.files.local.disk'));
        $file = UploadedFile::fake()->image('selfie.jpg');

        $response = $this->call('PUT', $url, [], [], [], [], $file->get());

        $response->assertStatus(403);
        Storage::disk(config('native-event-app.files.local.disk'))
            ->assertMissing(config('native-event-app.files.local.path') . '/' . $uuid . '.jpg');
    }

    /** @test */
    public function it_validates_the_request(): void
    {
        $uuid = (string) Str::uuid();
        $response = $this->call('PUT', $this->createSignedUrl($uuid), [], [], [], [], null);

        $response->assertStatus(422);
        $response->assertJsonStructure(['error']);
        $response->assertJsonMissingValidationErrors(['uuid', 'signature', 'expires']);
    }
}
