<?php

namespace Drp\NativeEventApp\Test\Unit\Templates;

use Carbon\Carbon;
use Drp\NativeEventApp\Templates\DataProviders\DataProvider;
use Drp\NativeEventApp\Templates\DataProviders\DataProviderRegistry;
use Drp\NativeEventApp\Test\TestCase;

class DataProviderRegistryTest extends TestCase
{
    /**
     * @var DataProviderRegistry
     */
    protected $registry;

    protected function setUp(): void
    {
        parent::setUp();

        $this->registry = new DataProviderRegistry();
    }

    /** @test */
    public function can_register_data_provider()
    {
        $provider = new class extends DataProvider {
        };

        $this->registry->bind('test', function () use ($provider): DataProvider {
            return $provider;
        });

        $this->assertSame($provider, $this->registry->get('test'));
    }

    /** @test */
    public function can_resolve_provider_within_a_provider()
    {
        $hotel = new class(['name' => 'Grand Budapest']) extends DataProvider {
            /** @var string */
            public $name;
        };
        $user = new class(['hotel' => $hotel]) extends DataProvider {
            /** @var mixed */
            public $hotel;
        };

        $this->registry->bind('user', function () use ($user) {
            return $user;
        });

        $this->assertEquals('Grand Budapest', $this->registry->resolve('user.hotel.name'));
    }

    /** @test */
    public function can_resolve_dates()
    {
        $user = new class(['startsAt' => Carbon::parse('1st January 2020')]) extends DataProvider {
            /** @var \Carbon\Carbon */
            public $startsAt;
        };

        $this->registry->bind('user', function () use ($user) {
            return $user;
        });

        $this->assertEquals('01/01/2020', $this->registry->resolve('user.startsAt.format(d/m/Y)'));
    }
}
